(* Berechnung von Mondphasen und Positionen, Sonne- und Mondfinsternissen

   auf der Basis von "Astronomische Algorithmen von Jean Meeus"
   siehe auch:
   https://www.computus.de/mondphase/mondphase

   bertragung nach Delphi: J. Rathlev
   Berechnete Zeiten in UTC
   August 2019
*)

unit Mondphasen;

interface

procedure MpTest;

implementation

uses System.SysUtils, System.Math, System.DateUtils;

const
  xDeg = Pi/180;
  xRad = 180/Pi;

function JDToDt (JD : double) : TDateTime;
begin
  Result:=JD+EncodeDate(1970,1,1)-2440587.5;
  end;

function DtToJD (dt : TDateTime) : double;
begin
  Result:=dt-EncodeDate(1970,1,1)+2440587.5-Result;
  end;

function Dezimaljahr : double;
var
  dt : TDateTime;
  j  : word;
begin
  dt:=Today; j:=YearOf(dt);
  Result:=j+DayOfTheYear(dt)/DaysInAYear(j);
  end;

function Var_k (tz : integer) : double;
var
  dt : TDateTime;
begin
  dt:=Today;
  dt:=EncodeDate(2022,4,30);
  Result:=(YearOf(dt)+(pred(MonthOf(dt))*30.4+DayOf(dt)+tz)/365-2000)*12.3685;
//  Result:=(Heute.getFullYear() + (Heute.getMonth() * 30.4 + Heute.getDate() + tz) / 365 - 2000) * 12.3685;
  end;

function Tagesdifferenz(JDE : double) : double;
  // aktuelles Datum
begin
  Result:=RoundTo(JDE - DtToJD(Today),-1);
  end;

function Var_o (k,t : double) : double;
begin
  Result:=124.7746-1.5637558*k+(0.0020691+0.00000215*t)*sqr(t);
  end;

function Var_f (k,t : double) : double;
begin
  Result:=160.7108+390.67050274*k+(- 0.0016341+(-0.00000227+0.000000011*t)*t)*sqr(t);
  end;

function Var_m1 (k,t : double) : double;
begin
  Result:=201.5643+385.81693528*k+(0.1017438+(0.00001239-0.000000058*t)*t)*sqr(t);
  end;

function Var_m (k,t : double) : double;
begin
  Result:=2.5534+29.10535669*k-(0.0000218+0.00000011*t)*sqr(t);
  end;

function Var_e (t : double) : double;
begin
  Result:=1-(0.002516 + 0.0000074*t)*t;
  end;

function Var_JDE (k,t : double) : double;
begin
  Result:=2451550.09765+29.530588853*k+(0.0001337+(-0.00000015+0.00000000073*t)*t)*sqr(t);
  end;

function CS(x : double) : double;
begin
  Result:=cos(xDeg*x);
  end;

function SN(x : double) : double;
begin
  Result:=sin(xDeg*x);
  end;

function Korrektur(JDE,t,k : double) : double;
begin
  //Zustzlichen Korrekturen
  Result:=JDE+0.000325*SN(299.77+0.107408*k-0.009173*sqr(t))
    +0.000165*SN(251.88+0.016321*k)+0.000164*SN(251.83+26.651886*k)+0.000126*SN(349.42+36.412478*k)
    +0.00011*SN(84.66+18.206239*k)+0.000062*SN(141.74+53.303771*k)+0.00006*SN(207.14+2.453732*k)
    +0.000056*SN(154.84+7.30686*k)+0.000047*SN(34.52+27.261239*k)+0.000042*SN(207.19+0.121824*k)
    +0.00004*SN(291.34+1.844379*k)+0.000037*SN(161.72+24.198154*k)+0.000035*SN(239.56+25.513099*k)
    +0.000023*SN(331.55+3.592518*k);
  end;

function Neumond(k : double) : double;
var
  e,m,m1,f,o,t : double;
begin
  k:=Floor(k);
  t:=k/1236.85;
  e:=Var_e(t);
  m:=Var_m(k,t);
  m1:=Var_m1(k,t);
  f:=Var_f(k,t);
  o:=Var_o(k,t);
  //Neumondkorrekturen
  Result:=Var_JDE(k,t)-0.4072*SN(m1)+0.17241*e*SN(m)+0.01608*SN(2*m1)+0.01039*SN(2*f)
    +0.00739*e*SN(m1-m)-0.00514*e*SN(m1+m)+0.00208*sqr(e)*SN(2*m)-0.00111*SN(m1-2*f)
    -0.00057*SN(m1+2*f)+0.00056*e*SN(2*m1+m)-0.00042*SN(3*m1)+0.00042*e*SN(m+2*f)
    +0.00038*e*SN(m-2*f)-0.00024*e*SN(2*m1-m)-0.00017*SN(o)-0.00007*SN(m1+2*m)
    +0.00004*SN(2*m1-2*f)+0.00004*SN(3*m)+0.00003*SN(m1+m-2*f)+0.00003*SN(2*m1+2*f)
    -0.00003*SN(m1+m+2*f)+0.00003*SN(m1-m+2*f)-0.00002*SN(m1-m-2*f)-0.00002*SN(3*m1+m)+0.00002*SN(4*m1);
  Result:=Korrektur(Result,t,k);
  end;

function Viertel(k,modus : double) : double;
var
  w,e,m,m1,f,o,t : double;
begin
  // modus =0.25:=Erstes Viertel, modus =0.75:=Letztes Viertel
  k:=Floor(k)+modus;
  t:=k/1236.85;
  e:=Var_e(t);
  m:=Var_m(k,t);
  m1:=Var_m1(k,t);
  f:=Var_f(k,t);
  o:=Var_o(k,t);
  // Viertelmondkorrekturen
  Result:=Var_JDE(k,t)-0.62801*SN(m1)+0.17172*e*SN(m)-0.01183*e*SN(m1+m)+0.00862*SN(2*m1)
    +0.00804*SN(2*f)+0.00454*e*SN(m1- m)+0.00204*e*e*SN(2*m)-0.0018*SN(m1- 2*f)-0.0007*SN(m1+2*f)
    -0.0004*SN(3*m1)-0.00034*e*SN(2*m1-m)+0.00032*e*SN(m+2*f)+0.00032*e*SN(m- 2*f)
    -0.00028*e*e*SN(m1+2*m)+0.00027*e*SN(2*m1+m)-0.00017*SN(o)-0.00005*SN(m1- m- 2*f)
    +0.00004*SN(2*m1+2*f)-0.00004*SN(m1+m+2*f)+0.00004*SN(m1-2*m)+0.00003*SN(m1+m-2*f)
    +0.00003*SN(3*m)+0.00002*SN(2*m1-2*f)+0.00002*SN(m1- m+2*f)-0.00002*SN(3*m1+m);
  w:=0.00306-0.00038*e*CS(m)+0.00026*CS(m1)-0.00002*CS(m1- m)+0.00002*CS(m1+m)+0.00002*CS(2*f);
  if (modus<0.5) then Result:=Result+w else Result:=Result-w;
  Result:=Korrektur(Result,t,k);
  end;

function Vollmond(k : double) : double;
var
  e,m,m1,f,o,t : double;
begin
  k:=Floor(k)+0.5;
  t:=k/1236.85;
  e:=Var_e(t);
  m:=Var_m(k,t);
  m1:=Var_m1(k,t);
  f:=Var_f(k,t);
  o:=Var_o(k,t);
  //Vollmondkorrekturen
  Result:=Var_JDE(k,t)-0.40614*SN(m1)+0.17302*e*SN(m)+0.01614*SN(2*m1)+0.01043*SN(2*f)
    +0.00734*e*SN(m1-m)-0.00515*e*SN(m1+m)+0.00209*e*e*SN(2*m)-0.00111*SN(m1-2*f)
    -0.00057*SN(m1+2*f)+0.00056*e*SN(2*m1+m)-0.00042*SN(3*m1)+0.00042*e*SN(m+2*f)
    +0.00038*e*SN(m-2*f)-0.00024*e*SN(2*m1-m)-0.00017*SN(o)-0.00007*SN(m1+2*m)+0.00004*SN(2*m1-2*f)
    +0.00004*SN(3*m)+0.00003*SN(m1+m- 2*f)+0.00003*SN(2*m1+2*f)-0.00003*SN(m1+m+2*f)
    +0.00003*SN(m1-m+2*f)-0.00002*SN(m1-m-2*f)-0.00002*SN(3*m1+m)+0.00002*SN(4*m1);
  Result:=Korrektur(Result,t,k);
  end;

type
  TEclipseType = (etSun,etMoon);
  TEclipseMode = (emPartial,emTotal,emAnnular);

function Finsternis (k : double; Typ : TEclipseType; Modus : TEclipseMode) : double;
var
  t,f,o,f1,a1,e,m,m1,p,q,g,u : double;
  Ringtest : boolean;
begin
  // Typ: Sonnenfinsternis,Mondfinsternis
  // Modus: Finsternis partiell, Finsternis total, Sonnenfinsternis ringfrmig
  k:=Floor(k);
  if Typ=etMoon then k:=k+0.5;
  t:=k/1236.85;
  f:=Var_f(k,t);
  Result:=0;
  if(SN(abs(f))<=0.36) then begin
    o:=Var_o(k,t);
    f1:=f-0.02665*SN(o);
    a1:=299.77+0.107408*k-0.009173*sqr(t);
    e:=Var_e(t);
    m:=Var_m(k,t);
    m1:=Var_m1(k,t);
    p:=0.207*e*SN(m)+0.0024*e*SN(2*m)-0.0392*SN(m1)+0.0116*SN(2*m1)-0.0073*e*SN(m1+m)
      +0.0067*e*SN(m1-m)+0.0118*SN(2*f1);
    q:=5.2207-0.0048*e*CS(m)+0.002*e*CS(2*m)-0.3299*CS(m1)-0.006*e*CS(m1+m)+0.0041*e*CS(m1-m);
    g:=(p*CS(f1)+q*SN(f1))*(1-0.0048*CS(abs(f1)));
    u:=0.0059+0.0046*e*CS(m)-0.0182*CS(m1)+0.0004*CS(2*m1)-0.0005*CS(m+m1);
    Result:=Var_JDE(k,t);
    if Typ=etMoon then Result:=Result-0.4065*SN(m1)+0.1727*e*SN(m)
    else Result:=Result-0.4075*SN(m1)+0.1721*e*SN(m);
    Result:=Result+0.0161*SN(2*m1)-0.0097*SN(2*f1)+0.0073*e*SN(m1-m)-0.005*e*SN(m1+m)
      -0.0023*SN(m1-2*f1)+0.0021*e*SN(2*m)+0.0012*SN(m1+2*f1)+0.0006*e*SN(2*m1+m)
      -0.0004*SN(3*m1)-0.0003*e*SN(m+2*f1)+0.0003*SN(a1)-0.0002*e*SN(m-2*f1)
      -0.0002*e*SN(2*m1-m)-0.0002*SN(o);
    if Typ=etMoon then begin
      if ((1.0248-u-abs(g))/0.545<=0) then Result:=0 // keine Mf
      else if (Modus=emPartial) and ((1.0128-u-abs(g))/0.545>0)
        and (sqr(0.4678-u)-sqr(g)>0) then Result:=0   // keine partielle Mf
      else if (Modus=emTotal) and (((1.0128-u-abs(g))/0.545<=0)
        xor (sqr(0.4678-u)-sqr(g)<=0)) then Result:=0; // keine totale Mf
      end
    else begin
      if(abs(g)>1.5433+u) then Result:=0  // keine SF
      else if (Modus=emPartial) then begin
        if ((abs(g)<=0.9972) or ((abs(g)>=0.9972) and (abs(g)<0.9972+abs(u)))) then Result:=0  // keine partielle Sf
        end
      else begin   // total
        if (abs(g)>0.9972) or ((abs(g)<0.9972) and (abs(g)>0.9972+abs(u))) then Result:=0  // keine ringfrmige oder totale SF
        else begin
          Ringtest:=(u>0.0047) or (u>=0.00464*sqrt(1-sqr(g))); // keine totale Sf
          if Ringtest and (Modus=emTotal) then Result:=0
          else if not Ringtest and (Modus=emAnnular) then Result:=0;
          end;
        end;
      end;
    end;
  end;

function Beleuchtung (JDE : double) : double;
var t,d,m,m1 : double;
begin
  t:=(JDE-2451545)/36525;
  d:=297.8502042+(445267.11151686+(-0.00163+(1/545868-1/113065000*t)*t)*t)*t;
  m:=357.5291092+(35999.0502909+(-0.0001536+1/24490000*t)*t)*t;
  m1:=134.9634114+(477198.8676313+(0.008997+(1/69699-1/14712000*t)*t)*t)*t;
  Result:=(1+CS(180-d-6.289*SN(m1)+2.1*SN(m)-1.274*SN(2*d-m1)-0.658*SN(2*d)
    -0.241*SN(2*m1)-0.110*SN(d)))/2*100;
  end;

function DeklinationNord (AsDegree : boolean) : double;
var
  JDA,k,t,d,m,m1,f,e : double;
begin
  JDA:=DtToJD(Today);
  k:=Floor((Dezimaljahr-2000.03)*13.3686);
  repeat
    t:=k/1336.86;
    d:=152.2029+333.0705546*k-(0.0004025-0.00000011*t)*sqr(t);
    m:=14.8591+26.9281592*k-(0.0000544+0.0000001*t)*sqr(t);
    m1:=4.6881+356.9562795*k+(0.0103126+0.00001251*t)*sqr(t);
    f:=325.8867+1.4467806*k-(0.0020708+0.00000215*t)*sqr(t);
    e:=1-(0.002516+0.0000074*t)*t;
    Result:=2451562.5897+27.321582241*k+(0.000100695-0.000000141*t)*sqr(t)
      +0.8975*CS(f)-0.4726*SN(m1)-0.103*SN(2*f)-0.0976*SN(2*d-m1)-0.0462*CS(m1-f)-0.0461*CS(m1+f)
      -0.0438*SN(2*d)+0.0162*SN(m)*e-0.0157*CS(3*f)+0.0145*SN(m1+2*f)+0.0136*CS(2*d-f)
      -0.0095*CS(2*d-m1-f)-0.0091*CS(2*d-m1+f)-0.0089*CS(2*d+f)+0.0075*SN(2*m1)
      -0.0068*SN(m1-2*f)+0.0061*CS(2*m1-f)-0.0047*SN(m1+3*f)-0.0043*SN(2*d-m-m1)*e
      -0.004*CS(m1-2*f) -0.0037*SN(2*d-2*m1)+0.0031*SN(f)+0.003*SN(2*d+m1)-0.0029*CS(m1+2*f)
      -0.0029*SN(2*d-m)*e-0.0027*SN(m1+f)+0.0024*SN(m-m1)*e-0.0021*SN(m1-3*f)
      +0.0019*SN(2*m1+f)+0.0018*CS(2*d-2*m1-f)+0.0018*SN(3*f)+0.0017*CS(m1+3*f)+0.0017*CS(2*m1)
      -0.0014*CS(2*d-m1)+0.0013*CS(2*d+m1+f)+0.0013*CS(m1)+0.0012*SN(3*m1+f)
      +0.0011*SN(2*d-m1+f)-0.0011*CS(2*d-2*m1)+0.001*CS(d+f)+0.001*SN(m+m1)*e
      -0.0009*SN(2*d-2*f)+0.0007*CS(2*m1+f)-0.0007*CS(3*m1+f);
    k:=k+1;
    until (Result>=JDA);
  if AsDegree then Result:=23.6961-0.013004*t+5.1093*SN(f)+0.2658*CS(2*f)+0.1448*SN(2*d-f)
    -0.0322*SN(3*f)+0.0133*CS(2*d-2*f)+0.0125*CS(2*d)-0.0124*SN(m1-f)-0.0101*SN(m1+2*f)+0.0097*CS(f)
    -0.0087*SN(2*d+m-f)*e+0.0074*SN(m1+3*f)+0.0067*SN(d+f)+0.0063*SN(m1-2*f)+0.006*SN(2*d-m-f)*e
    -0.0057*SN(2*d-m1-f)-0.0056*CS(m1+f)+0.0052*CS(m1+2*f)+0.0041*CS(2*m1+f)
    -0.004*CS(m1-3*f)+0.0038*CS(2*m1-f)-0.0034*CS(m1-2*f)-0.0029*SN(2*m1)+0.0029*SN(3*m1+f)
    -0.0028*CS(2*d+m-f)*e-0.0028*CS(m1-f)-0.0023*CS(3*f)-0.0021*SN(2*d+f)+0.0019*CS(m1+3*f)
    +0.0018*CS(d+f)+0.0017*SN(2*m1-f)+0.0015*CS(3*m1+f)+0.0014*CS(2*d+2*m1+f)-0.0012*SN(2*d-2*m1-f)
    -0.0012*CS(2*m1)-0.001*CS(m1)-0.001*SN(2*f)+0.0006*SN(m1+f);
  end;

function DeklinationSued(AsDegree : boolean) : double;
var
  JDA,k,t,d,m,m1,f,e : double;
begin
  JDA:=DtToJD(Today);
  k:=Floor((Dezimaljahr-2000.03)*13.3686);
  repeat
    t:=k/1336.86;
    d:=345.6676+333.0705546*k-(0.0004025-0.00000011*t)*sqr(t);
    m:=1.3951+26.9281592*k-(0.0000544+0.0000001*t)*sqr(t);
    m1:=186.21+356.9562795*k+(0.0103126+0.00001251*t)*sqr(t);
    f:=145.1633+1.4467806*k-(0.0020708+0.00000215*t)*sqr(t);
    e:=1-(0.002516+0.0000074*t)*t;
    Result:=2451548.9289+27.321582241*k+(0.000100695-0.000000141*t)*sqr(t)
      -0.8975*CS(f)-0.4726*SN(m1)-0.103*SN(2*f)-0.0976*SN(2*d-m1)+0.0541*CS(m1-f)+0.0516*CS(m1+f)
      -0.0438*SN(2*d)+0.0112*SN(m)*e+0.0157*CS(3*f)+0.0023*SN(m1+2*f)-0.0136*CS(2*d-f)
      +0.011*CS(2*d-m1-f)+0.0091*CS(2*d-m1+f)+0.0089*CS(2*d+f)+0.0075*SN(2*m1)-0.003*SN(m1-2*f)
      -0.0061*CS(2*m1-f)-0.0047*SN(m1+3*f)-0.0043*SN(2*d-m-m1)*e
      +0.004*CS(m1-2*f) -0.0037*SN(2*d-2*m1)-0.0031*SN(f)+0.003*SN(2*d+m1)+0.0029*CS(m1+2*f)
      -0.0029*SN(2*d-m)*e-0.0027*SN(m1+f)+0.0024*SN(m-m1)*e-0.0021*SN(m1-3*f)
      -0.0019*SN(2*m1+f)-0.0006*CS(2*d-2*m1-f)-0.0018*SN(3*f)-0.0017*CS(m1+3*f)+0.0017*CS(2*m1)
      +0.0014*CS(2*d-m1)-0.0013*CS(2*d+m1+f)-0.0013*CS(m1)+0.0012*SN(3*m1+f)
      +0.0011*SN(2*d-m1+f)+0.0011*CS(2*d-2*m1)+0.001*CS(d+f)+0.001*SN(m+m1)*e-0.0009*SN(2*d-2*f)
      -0.0007*CS(2*m1+f)-0.0007*CS(3*m1+f);
    k:=k+1;
    until (Result>=JDA);
  if AsDegree then Result:=23.6961-0.013004*t-5.1093*SN(f)+0.2658*CS(2*f)-0.1448*SN(2*d-f)
    +0.0322*SN(3*f)+0.0133*CS(2*d-2*f)+0.0125*CS(2*d)-0.0015*SN(m1-f)+0.0101*SN(m1+2*f)-0.0097*CS(f)
    +0.0087*SN(2*d+m-f)*e+0.0074*SN(m1+3*f)+0.0067*SN(d+f)-0.0063*SN(m1-2*f)-0.006*SN(2*d-m-f)*e
    +0.0057*SN(2*d-m1-f)-0.0056*CS(m1+f)-0.0052*CS(m1+2*f)-0.0041*CS(2*m1+f)
    -0.004*CS(m1-3*f)+0.0038*CS(2*m1-f)+0.0034*CS(m1-2*f)-0.0029*SN(2*m1)+0.0029*SN(3*m1+f)
    +0.0028*CS(2*d+m-f)*e-0.0028*CS(m1-f)+0.0023*CS(3*f)+0.0021*SN(2*d+f)+0.0019*CS(m1+3*f)
    +0.0018*CS(d+f)-0.0017*SN(2*m1-f)+0.0015*CS(3*m1+f)+0.0014*CS(2*d+2*m1+f)+0.0012*SN(2*d-2*m1-f)
    -0.0012*CS(2*m1)+0.001*CS(m1)-0.001*SN(2*f)+0.0037*SN(m1+f);
  end;

function Mondknoten : double;
var
  JDA,k,t,d,m,m1,o,v,p,e : double;
begin
  JDA:=DtToJD(Today);
  k:=Floor((Dezimaljahr-2000.05)*13.4223);
  repeat
    t:=k/1342.23;
    d:=183.638+331.73735691*k+(0.0015057+(0.00000209-0.00000001*t)*t)*sqr(t);
    e:=1-(0.002516+0.0000074*t)*t;
    m:=17.4006+26.82037250*k+(0.0000999+0.00000006*t)*sqr(t);
    m1:=38.3776+355.52747322*k+(0.0123577+(0.000014628-0.000000069*t)*t)*sqr(t);
    o:=123.9767-1.44098949*k+(0.0020625+(0.00000214-0.000000016*t)*t)*sqr(t);
    v:=(299.75+132.85-0.009173*t)*t;
    p:=o+272.75-2.3*t;
    Result:=2451565.1619+27.212220817*k+(0.0002572+(0.000000021-0.000000000088*t)*t)*sqr(t)
      -0.4721*SN(m1)-0.1649*SN(2*d)-0.0868*SN(2*d-m1)+0.0084*SN(2*d+m1)-0.0083*SN(2*d-m)*e
      -0.0039*SN(2*d-m-m1)*e+0.0034*SN(2*m1)-0.0031*SN(2*d-2*m1)+0.003*SN(2*d+m)*e+0.0028*SN(m-m1)*e
      +0.0026*SN(m)+0.0025*SN(4*d)+0.0024*SN(d)+0.0022*SN(m+m1)*e
      +0.0017*SN(o)+0.0014*SN(4*d-m1)+0.0005*SN(2*d+m-m1)*e+0.0004*SN(2*d-m+m1)*e
      -0.0003*SN(2*d-2*m)*e+0.0003*SN(4*d-m)*e+0.0003*SN(v)+0.0003*SN(p);
    k:=k+0.5;
    until (Result>=JDA);
  if IsZero(k-Floor(k),0.01) then Result:=-Result; //absteigend
  end;

function Apogaeum : double;
var
  JDA,k,t,d,m,f : double;
begin
  JDA:=DtToJD(Today);
  k:=Floor((Dezimaljahr-1999.97)*13.2555)+0.5;
  repeat
    t:=k/1325.55;
    d:=171.9179+335.9106046*k+(-0.010025+(-0.00001156+0.000000055*t)*t)*sqr(t);
    m:=347.3477+27.1577721*k-(0.0008323+0.000001*t)*sqr(t);
    f:=316.6109+364.5287911*k-(0.0125131+0.0000148*t)*sqr(t);
    Result:=2451534.6698+27.55454988*k-(0.0006886-0.0000000052+0.000001098*t)*sqr(t)
      +0.4392*SN(2*d)+0.0684*SN(4*d)+(0.0456-0.00011*t)*SN(m)
      +(0.0426-0.00011*t)*SN(2*d-m)+0.0212*SN(2*f)
      -0.0189*SN(d)+0.0144*SN(6*d)+0.0113*SN(4*d-m)+0.0047*SN(2*d+2*f)+0.0036*SN(d+m)
      +0.0035*SN(8*d)+0.0034*SN(6*d-m)-0.0034*SN(2*d-2*f)+0.0022*SN(2*d-2*m)-0.0017*SN(3*d)
      +0.0013*SN(4*d+2*f)+0.0011*SN(8*d-m)+0.001*SN(4*d-2*m)+0.0009*SN(10*d)+0.0007*SN(3*d+m)
      +0.0006*SN(2*m)+0.0005*SN(2*d+m)+0.0005*SN(2*d+2*m)+0.0004*SN(6*d+2*f)
      +0.0004*SN(6*d-2*m)+0.0004*SN(10*d-m)-0.0004*SN(5*d)-0.0004*SN(4*d-2*f)
      +0.0003*SN(2*f+m)+0.0003*SN(12*d)+0.0003*SN(2*d+2*f-m)-0.0003*SN(d-m);
    k:=k+1;
    until (Result>=JDA);
  end;

function Perigaeum : double;
var
  JDA,k,t,d,m,f : double;
begin
  JDA:=DtToJD(Today);
  k:=Floor((Dezimaljahr-1999.97)*13.2555);
  repeat
    t:=k/1325.55;
    d:=171.9179+335.9106046*k+(-0.010025+(-0.00001156+0.000000055*t)*t)*sqr(t);
    m:=347.3477+27.1577721*k-(0.0008323+0.000001*t)*sqr(t);
    f:=316.6109+364.5287911*k-(0.0125131+0.0000148*t)*sqr(t);
    Result:=2451534.6698+27.55454988*k-(0.0006886-0.0000000052+0.000001098*t)*sqr(t)
      -1.6769*SN(2*d)+0.4589*SN(4*d)-0.1856*SN(6*d)+0.0883*SN(8*d)
      -(0.0773+0.00019*t)*SN(2*d-m)+(0.0502-0.00013*t)*SN(m)-0.046*SN(10*d)
      +(0.0422-0.00011*t)*SN(4*d-m)-0.0256*SN(6*d-m)+0.0253*SN(12*d)+0.0237*SN(d)
      +0.0162*SN(8*d-m)-0.0145*SN(14*d)+0.0129*SN(2*f)-0.0112*SN(3*d)-0.0104*SN(10*d-m)
      +0.0086*SN(16*d)+0.0069*SN(12*d-m)+0.0066*SN(5*d)-0.0053*SN(2*d+2*f)
      -0.0052*SN(18*d)-0.0046*SN(14*d-m)-0.0041*SN(7*d)+0.004*SN(2*d+m)+0.0032*SN(20*d)
      -0.0032*SN(d+m)+0.0031*SN(16*d-m)-0.0029*SN(4*d+m)-0.0027*SN(2*d-2*m)+0.0024*SN(4*d-2*m)
      -0.0021*SN(6*d-2*m)-0.0021*SN(22*d)-0.0021*SN(18*d-m)
      +0.0019*SN(6*d+m)-0.0018*SN(11*d)-0.0014*SN(8*d+m)-0.0014*SN(4*d-2*f)-0.0014*SN(6*d-2*f)
      +0.0014*SN(3*d+m)-0.0014*SN(5*d+m)+0.0013*SN(13*d)
      +0.0013*SN(20*d-m)+0.0011*SN(3*d+2*m)-0.0011*SN(4*d+2*f-2*m)-0.001*SN(d+2*m)
      -0.0009*SN(22*d-m)-0.0008*SN(4*f)+0.0008*SN(6*d-2*f)+0.0008*SN(2*d-2*f+m)
      +0.0007*SN(2*m)+0.0007*SN(2*f-m)+0.0007*SN(2*d+4*f)-0.0006*SN(2*f-2*m)-0.0006*SN(2*d-2*f+2*m)
      +0.0006*SN(24*d)+0.0005*SN(4*d-4*f)+0.0005*SN(2*d+2*m)-0.0004*SN(d-m)+0.0027*SN(9*d)+0.0027*SN(4*d+2*f);
    k:=k+1;
    until (Result>=JDA);
  end;

function Koeffizient(d,m,m1,f : double) : double;
var
  t : integer;
type
  TDataArr = array[0..59] of integer;
const
  kd : TDataArr = (0,2,2,0,0,0,2,2,2,2,0,1,0,2,0,0,4,0,4,2,2,1,1,2,2,4,2,0,2,2,1,2,0,0,
                   2,2,2,4,0,3,2,4,0,2,2,2,4,0,4,1,2,0,1,3,4,2,0,1,2,2);
  km : TDataArr = (0,0,0,0,1,0,0,-1,0,-1,1,0,1,0,0,0,0,0,0,1,1,0,1,-1,0,0,0,1,0,-1,0,-2,
                   1,2,-2,0,0,-1,0,0,1,-1,2,2,1,-1,0,0,-1,0,1,0,1,0,0,-1,2,1,0,0);
  km1 : TDataArr = (1,-1,0,2,0,0,-2,-1,1,0,-1,0,1,0,1,1,-1,3,-2,-1,0,-1,0,1,2,0,-3,-2,-1,
                   -2,1,0,2,0,-1,1,0,-1,2,-1,1,-2,-1,-1,-2,0,1,4,0,-2,0,2,1,-2,-3,2,1,-1,3,-1);
  kf : TDataArr = (0,0,0,0,0,2,0,0,0,0,0,0,0,-2,2,-2,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,-2,
                   2,0,2,0,0,0,0,0,0,-2,0,0,0,0,-2,-2,0,0,0,0,0,0,0,-2);
  kr : TDataArr = (-20905355,-3699111,-2955968,-569925,48888,-3149,246158,-152138,-170733,
                   -204586,-129620,108743,104755,10321,0,79661,-34782,-23210,-21636,24208,
                   30824,-8379,-16675,-12831,-10445,-11650,14403,-7003,0,10056,6322,-9884,
                   5751,0,-4950,4130,0,-3958,0,3258,2616,-1897,-2117,2354,0,0,-1423,-1117,
                   -1571,-1739,0,-4421,0,0,0,0,1165,0,0,8752);
begin
  Result:=0;
  for t:=0 to 59 do
    Result:=Result+kr[t]*CS(kd[t]*d+km[t]*m+km1[t]*m1+kf[t]*f);
  end;

function Entfernung (JD : double) : double;
var
  t,d,m,m1,f : double;
begin
  t:=(JD-2451545)/36525;
  d:=297.8502042+(445267.11151686+(-0.00163+(1/545868-1/113065000*t)*t)*t)*t;
  m:=357.5291092+(35999.0502909-(0.0001536-1/24490000*t)*t)*t;
  m1:=134.9634114+(477198.8676313+(0.008997+(1/69699-1/14712000*t)*t)*t)*t;
  f:=93.27209929999999+(483202.0175273+(-0.0034029+(-1/3526000+1/863310000*t)*t)*t)*t;
  Result:=385000.56+Koeffizient(d,m,m1,f)/1000;
  end;

function NaechsterVM (zeit : double) : double;
var
  tz : integer;
  k  : double;
begin
  tz:=0;
  repeat
    k:=Vollmond(Var_k(tz));
    inc(tz);
    until (k>=zeit);
  Result:=k;
  end;

function NaechstesLV (zeit : double) : double;
var
  tz : integer;
  k  : double;
begin
  tz:=0;
  repeat
    k:=Viertel(Var_k(tz),0.75);
    inc(tz);
    until (k>=zeit);
  Result:=k;
  end;

function NaechsterNM (zeit : double) : double;
var
  tz : integer;
  k  : double;
begin
  tz:=0;
  repeat
    k:=Neumond(Var_k(tz));
    inc(tz);
    until (k>=zeit);
  Result:=k;
  end;

function NaechstesEV (zeit : double) : double;
var
  tz : integer;
  k  : double;
begin
  tz:=0;
  repeat
    k:=Viertel(Var_k(tz),0.25);
    inc(tz);
    until (k>=zeit);
  Result:=k;
  end;

function NaechsteMF (zeit : double; Modus : TEclipseMode) : double;
var
  tz : integer;
  k  : double;
begin
  tz:=0;
  repeat
    k:=Finsternis(Var_k(tz),etMoon,Modus);
    inc(tz);
    until (k>=zeit);
  Result:=k;
  end;

function NaechsteSF (zeit : double; Modus : TEclipseMode) : double;
var
  tz : integer;
  k  : double;
begin
  tz:=0;
  repeat
    k:=Finsternis(Var_k(tz),etSun,Modus);
    inc(tz);
    until (k>=zeit);
  Result:=k;
  end;

procedure MpTest;
var
  x,zeit : double;
  dt     : TDateTime;
begin
  zeit:=DtToJD(Now);
  x:=Entfernung(Apogaeum);
  x:=Entfernung(Perigaeum);
  dt:=JDToDt(Perigaeum);
  dt:=JDToDt(Apogaeum);
  x:=Mondknoten;
  dt:=JDToDt(abs(x));
//  dt:=JDToDt(DeklinationSued(false));
//  x:=DeklinationSued(true);
//  dt:=JDToDt(DeklinationNord(false));
//  x:=DeklinationNord(true);
//  x:=Beleuchtung(2458701.978);
//  dt:=JDToDt(NaechsterNM(zeit));
//  dt:=JDToDt(NaechstesLV(zeit));
//  dt:=JDToDt(NaechsterVM(zeit));
//  dt:=JDToDt(NaechstesEV(zeit));
//  dt:=JDToDt(NaechsteSF(zeit,emAnnular));
//  dt:=JDToDt(NaechsteSF(zeit,emTotal));
//  dt:=JDToDt(NaechsteSF(zeit,emPartial));
//  dt:=JDToDt(NaechsteMF(zeit,emTotal));
//  dt:=JDToDt(NaechsteMF(zeit,emPartial));

  end;

end.
